function [J, grad] = costFunctionReg(theta, X, y, lambda)

%   COSTFUNCTIONREG Compute cost and gradient for logistic regression with regularization
%   J = COSTFUNCTIONREG(theta, X, y, lambda) computes the cost of using
%   theta as the parameter for regularized logistic regression and the
%   gradient of the cost w.r.t. to the parameters. 

m = size(X,1); % number of training examples

Xt = X*theta; % (m,1)
Htheta_X = sigmoid(Xt); % (m,1)

J = -1/m*sum(y.*log(Htheta_X) + (ones(m,1)-y).*log(ones(m,1) - Htheta_X)) + (lambda/2/m)*(theta(2:end)'*theta(2:end));

grad = (X'*(Htheta_X-y))/m;% (n+1 , 1) number of features
grad(2:end) = grad(2:end) + (lambda/m)*theta(2:end);

end
